C
C *********************************************************************
C
C                              P A T H 3 D
C             A Ground-Water Path and Travel-Time Simulator
C
C                              By C. Zheng
C                  S.S. Papadopulos & Associates, Inc.
C                         7944 Wisconsin Avenue
C                     Bethesda, Maryland 20814, USA
C                                Now at
C                         University of Alabama
C                         Department of Geology
C                          Tuscaloosa, AL 35487
C
C **********************************************************************
C V4.00: 10-Jun-1994
c v4.01: CJN 96/02
c V4.50: January 2001
c          adds support for BINARY headsave files
c          uses ALLOCATABLE arrays
c          expands initial particle placement options
c          corrects particle removal at RIV, STR, and GHB
c
C v4.6:  January 2001
c          allow particles to be released at user-defined time or stress period
c          allow reverse transient tracking
c
c  v4.6    PARAMETER (LENY=09999999)
c  v4.6    PARAMETER (NFMAX=200)
      PARAMETER (INBAS=1,IOUT=6,
     &           INTRK=2,INH=3,INPLOT=4,IFRONT=7,ICNFG=8,ICAPT=9)
      CHARACTER HDFORM*80,HPRT*1,FLNAME*50,FINDEX*30
      DIMENSION TFRONT(:),IUNIT(24)                                     !v4.6
      allocatable :: tfront                                             !v4.6
      integer,allocatable :: nsper(:,2)                                 !v4.6 for starting stress period
      real, allocatable  :: reltime(:,2)                                !v4.6 for starting time
      dimension y(:)                                                    !v4.6 make Y-array allocatable
      allocatable :: y
      integer ierr
      COMMON /UNIT/INBCF,INWEL,INDRN,INRIV,INEVT,INGHB,INRCH,INSTR
      COMMON /CONST/ROERR,TINY,IOPSS,SAFETY
      COMMON /LC/LAYCON(80)
      COMMON /VP/JP,IP,KP
      COMMON /STR/NSS,NTRIB,NDIV,ICALC
      COMMON /DRY/HDRY,IWDFLG
C
C--INITIALIZE GLOBAL CONSTANTS
      ROERR=1.E-6
      TINY=1.E-30
      SAFETY=0.90
C
C--WRITE AN IDENTIFIER TO SCREEN
      WRITE(*,101)
  101 FORMAT(1X,'Path3D--A Groundwater Path and Travel-Time Simulator',
     &'  V4.60'
     &/1X,'Copyright (c)1989-2001  S.S.Papadopulos & Associates, Inc.'/)
C
C--OPEN OUTPUT FILE AND PRINT PROGRAM TITLE AND VERSION NUMBER
      FINDEX='Standard Output File: '
      CALL OPENFL(IOUT,3,0,FLNAME,0,FINDEX)
      WRITE(IOUT,11)
   11 FORMAT(/38X,53('+')/38X,'+',51X,'+'/38X,'+',20X,'P A T H 3 D',20X,
     & '+'/38X,'+',3X,'A Ground-Water Path and Travel-Time Simulator',
     & 3X,'+'/38X,'+',21X,'(V. 4.60)',21X,'+'/38X,'+',51X,'+'
     & /38X,53('+')/)
C
C--OPEN PARTICLE TRACKING INPUT FILE
      FINDEX='Particle Tracking Input File: '
      CALL OPENFL(INTRK,3,1,FLNAME,0,FINDEX)
C
C--OPEN INPUT HEAD
   15 WRITE(*,*) 'Enter Name of Input Head File: '
      READ(*,'(A50)') FLNAME
      IF(FLNAME.EQ.' ') GOTO 15
      WRITE(*,*) 'Enter U if the head file is UNFORMATTED;'
      write(*,*) 'Enter B in the head file is TRUE BINARY;'             !v4.6
      WRITE(*,*) 'Otherwise, enter format of the head file: '
      READ(*,'(A80)') HDFORM
      J=INDEX(HDFORM,' ')-1
      IF(HDFORM(1:J).EQ.'U'.OR.HDFORM(1:J).EQ.'u') THEN                 !v4.6 Unformatted
        CALL OPENFL(INH,1,1,FLNAME,1,FINDEX)
      ELSEif(HDFORM(1:J).eq.'B'.or.HDFORM(1:J).eq.'b') then             !v4.6 Binary
        CALL OPENFL(INH,2,1,FLNAME,1,FINDEX)
      ELSE
        CALL OPENFL(INH,3,1,FLNAME,1,FINDEX)                            !v4.6 Formatted
      ENDIF
      WRITE(*,*) 'Print out heads for checking? (Enter Y or N) '
      READ(*,'(A1)') HPRT
      IF(HPRT.EQ.' ') HPRT='N'
C
C--CALL SUBROUTIN GETDIM TO ALLOCATE SPACE FOR ALL DATA ARRAYS
      ISUM=1
      CALL GETDIM(INBAS,INTRK,IOUT,INPLOT,ISUM,NCOL,NROW,NLAY,NPER,
     & IUNIT,LCIB,LCH,LCKW,ISS,LCDELR,LCDELC,LCXBC,LCYBC,LCZBC,LCTRPY,
     & LCHY,LCVCON,LCHTOP,LCDZ,LCPR,MXWEL,LCWEL,MXDRN,LCDRN,MXRIV,
     & LCRIV,MXSTR,LCSTR,MXGHB,LCGHB,NRCHOP,LCIRCH,LCRECH,NEVTOP,
     & LCIEVT,LCEVTR,LCEXDP,LCSURF,LCQX,LCQY,LCQZ,NCOUNT,NTRKOP,
     & INIPOS,LCPP,LCPI,IFRONT,NFRONT,ICAPT,INCAPT,itime)               !v4.6 del NFMAX, add ITIME
c
c v4.6 allocate arrays for info on particle release times/periods
      if (itime.ne.0) then                                              !v4.6
          allocate(nsper(ncount,2),reltime(ncount,2))                   !v4.6
      endif                                                             !v4.6
C
C--CHECK WHETHER Y ARRAY IS DIMENSIONED LARGE ENOUGH
      ISUM1=ISUM
      WRITE(IOUT,16) ISUM1
   16 FORMAT(1X,I10,' ELEMENTS IN THE Y ARRAY')                         !v4.6
C
      allocate (y(isum1),stat=ierr)
      if(ierr.ne.0) then
        write(*,*) 'STOP.  Error allocating Y array.'
        stop
      endif
      allocate (tfront(nfront),stat=ierr)
      if(ierr.ne.0) then
        write(*,*) 'STOP.  Error allocating TFRONT array.'
        stop
      endif
c
C--INITIALIZE ELEMENTS OF THE Y ARRAY TO BE USED
      DO 19 I=1,ISUM1
        Y(I)=0
   19 CONTINUE
C
C--CALL SUBROUTINE GETDAT TO READ AND PREPARE INPUT DATA
C--FOR ENTIRE SIMULATION
      CALL GETDAT(INBAS,INBCF,INTRK,IOUT,NCOL,NROW,NLAY,
     & Y(LCIB),Y(LCH),ISS,Y(LCDELR),Y(LCDELC),Y(LCXBC),Y(LCYBC),
     & Y(LCZBC),Y(LCTRPY),Y(LCHY),Y(LCVCON),Y(LCHTOP),Y(LCDZ),
     & Y(LCPR),XMAX,YMAX,ZMAX,NCOUNT,TIME1,TIME2,EPS,DT,MAXSTP,
     & ISAV,IPRT,Y(LCPP),Y(LCPI),NFRONT,IFRONT,TFRONT,                  !v4.6 del NFMAX
     & NTRKOP,INIPOS,INPLOT,ICNFG,ICAPT,INCAPT,HNOFLO,itime,
     + nsper,reltime)                                                   !v4.6 add ITIME,NSP,PTIME
c-----------------------------------------------------------------------
c v4.6   check whether tracking is forward or backward
      if (itime .eq. 0) then
          if (iss .eq. 0) then
              if (time1 .lt. time2) then
                  write(iout,*) 'FORWARD TRACKING'
                  idirection=1
              else if (time2 .lt. time1) then
                  write(iout,*) 'BACKWARD TRACKING'
                  idirection=-1
              endif
          endif
      else if (itime .gt. 0) then
          idirection=0
          ifirsttime=0
          do n=1,ncount
              if(reltime(n,1).lt.reltime(n,2)) then
                  idirection=1
              else if (reltime(n,1).gt.reltime(n,2)) then
                  idirection=-1
              end if
              if (ifirsttime .ne. 0) then
                  if (idirectionold .ne. idirection) then
                      write(*,*) 'Cannot mix forward and'//
     +                      ' backward tracking for particles.'
                      stop
                  endif
              endif
              ifirsttime=1
              idirectionold=idirection
          end do
          if (idirection.eq.1) then
              write(iout,*) 'FORWARD TRACKING'
          else
              write(iout,*) 'BACKWARD TRACKING'
          endif
      else
          idirection=0
          ifirsttime=0
          do n=1,ncount
              if(nsper(n,1).lt.nsper(n,2)) then
                  idirection=1
              else if (nsper(n,1).gt.nsper(n,2)) then
                  idirection=-1
              end if
              if (ifirsttime .ne. 0) then
                  if (idirectionold .ne. idirection) then
                      write(*,*) 'Cannot mix forward and'//
     +                       ' backward tracking for particles.'
                      stop
                  endif
              endif
              ifirsttime=1
              idirectionold=idirection
          end do
          if (idirection.eq.1) then
              write(iout,*) 'FORWARD TRACKING'
          else
              write(iout,*) 'BACKWARD TRACKING'
          endif
      endif
c end v4.6 section here
c-----------------------------------------------------------------------


      if (idirection .eq.1) then
C--FOR EACH STRESS PERIOD
      NRC=0
      NF=1
      HT1=0.0
      HT2=0.0
      DO 1000 KPER=1,NPER
          WRITE(*,20) KPER
          WRITE(IOUT,20) KPER
   20     FORMAT(/29X,'STRESS PERIOD NO.',I5/20X,40('-'))
C
C--IF TRANSIENT SIMULATION READ AND PRINT OUT TIMING INFORMATION
          IF(ISS.EQ.0) THEN
              READ(INBAS,'(F10.0,I10.0,F10.0)') PERLEN,NSTP,TSMULT
              WRITE(IOUT,22) PERLEN,NSTP,TSMULT
   22         FORMAT(/1X,'LENGTH OF THIS STRESS PERIOD =',G15.7,
     &         /1X,'NUMBER OF TIME STEPS FOR HEAD SOLUTION =',I10,
     &         /1X,'TIME STEP MULTIPLIER =',G15.7)
              IF(TSMULT.EQ.1) THEN
                  DELT=PERLEN/FLOAT(NSTP)
              ELSE
                  DELT=PERLEN*(1.-TSMULT)/(1.-TSMULT**NSTP)
              ENDIF
          ELSE
              NSTP=1
          ENDIF
C
C--READ AND PREPARE SINK/SOURCE DATA FOR EACH STRESS PERIOD
      CALL GETSS(IOUT,NCOL,NROW,MXWEL,NWELLS,Y(LCWEL),MXDRN,
     & NDRAIN,Y(LCDRN),MXRIV,NRIVER,Y(LCRIV),MXSTR,NSTREM,Y(LCSTR),
     & MXGHB,NBOUND,Y(LCGHB),NRCHOP,Y(LCIRCH),Y(LCRECH),
     & NEVTOP,Y(LCIEVT),Y(LCEVTR),Y(LCEXDP),Y(LCSURF))
C
C--FOR EACH TIME STEP
      DO 900 KSTP=1,NSTP
          IF(KSTP.GT.1) DELT=DELT*TSMULT
          HT1=HT2
          HT2=HT2+DELT
          WRITE(*,30) KSTP
          WRITE(IOUT,30) KSTP
          IF(ISS.EQ.0) THEN
              WRITE(IOUT,31) HT1,HT2
          ENDIF
   30     FORMAT(/1X,'TIME STEP NO.',I5/1X,18('-'))
   31     FORMAT(1X,'FROM TIME =',G13.5,' TO ',G13.5)
C
C--READ IN HEAD DATA FOR EACH TIME STEP
          CALL READH(INH,IOUT,NCOL,NROW,NLAY,Y(LCIB),Y(LCH),HNOFLO,
     &                     HPRT,HDFORM)
C
C--IF STEADY STATE SIMULATION, MOVE ALL PARTICLES DIRECTLY FROM
C--START (TIME1) TO END (TIME2).
          IF(ISS.NE.0) THEN
              TSTART=TIME1
              TEND=TIME2
              GOTO 100
          ENDIF
c v4.6 - if ITIME option not invoked, then can track particles as done
c        previously, otherwise, need to check particles individually
c        within MOVE subroutine
          if (itime .eq. 0) then                                        !v4.6
C
C--OTHERWISE, MOVE PARTICLES BETWEEN TIME STEPS OF THE HEAD SOLUTION
              IF(TIME1.GT.HT2) THEN
                  WRITE(IOUT,41)
   41   FORMAT(/1X,'>>NO PARTICLES EXIST FOR TRACKING WITHIN',
     &  ' CURRENT TIME STEP')
                  GOTO 900
              ELSEIF(TIME1.GT.HT1.AND.TIME2.LT.HT2) THEN
                  TSTART=TIME1
                  TEND=TIME2
              ELSEIF(TIME1.LE.HT1.AND.TIME2.GT.HT1.AND.TIME2.LT.HT2)THEN
                  TSTART=HT1
                  TEND=TIME2
              ELSEIF(TIME1.GT.HT1.AND.TIME1.LT.HT2.AND.TIME2.GE.HT2)THEN
                  TSTART=TIME1
                  TEND=HT2
              ELSE
                  TSTART=HT1
                  TEND=HT2
              ENDIF
          else                                                          !v4.6
                  tstart=ht1                                            !v4.6
                  tend=ht2                                              !v4.6
          end if                                                        !v4.6
C
C--CALCULATE INTERNODAL DARCY VELOCITIES
  100     CALL VFACE(NCOL,NROW,NLAY,Y(LCIB),Y(LCH),Y(LCKW),
     &    Y(LCDELR),Y(LCDELC),Y(LCHY),Y(LCTRPY),Y(LCVCON),
     &    MXGHB,NBOUND,Y(LCGHB),NRCHOP,Y(LCIRCH),Y(LCRECH),
     &    NEVTOP,Y(LCIEVT),Y(LCEVTR),Y(LCEXDP),Y(LCSURF),
     &    Y(LCQX),Y(LCQY),Y(LCQZ),idirection)
C
C--MOVE PARTICLES
          CALL MOVE(IOUT,INPLOT,NCOUNT,NRC,NTRKOP,Y(LCPP),Y(LCPI),
     &    TSTART,TEND,EPS,DT,MAXSTP,ISAV,IPRT,NCOL,NROW,NLAY,Y(LCIB),
     &    Y(LCH),Y(LCKW),Y(LCDELR),Y(LCDELC),Y(LCXBC),Y(LCYBC),Y(LCZBC),
     &    Y(LCHTOP),Y(LCDZ),Y(LCPR),XMAX,YMAX,ZMAX,Y(LCQX),Y(LCQY),
     +    Y(LCQZ),ISS,NBOUND,MXGHB,Y(LCGHB),NRIVER,MXRIV,Y(LCRIV),
     +    NSTREM,MXSTR,Y(LCSTR),NDRAIN,MXDRN,Y(LCDRN),NWELLS,MXWEL,
     +    Y(LCWEL),IFRONT,NFRONT,TFRONT,NF,ICAPT,INCAPT,itime,          !v4.6 del NFMAX
     +    nsper,reltime,kper,idirection)                                !v4.6
C
C--CHECK TO SEE WHETHER ALL PARTICLES HAVE ALREADY BEEN REMOVED
C--OR WHETHER IT IS END OF TRACKING TIME INTERVAL
          IF(NRC.EQ.NCOUNT) THEN
              WRITE(IOUT,43)
   43         FORMAT(/1X,'>>ALL PARTICLES HAVE ALREADY BEEN REMOVED')
              GOTO 1001
          ELSEIF(ABS(TEND-TIME2).LT.ROERR) THEN
              WRITE(IOUT,42)
   42         FORMAT(/1X,'>>ALL PARTICLES HAVE ALREADY BEEN MOVED',
     &        ' TO END OF TRACKING TIME INTERVAL')
              GOTO 1001
          ENDIF
C
  900 CONTINUE
 1000 CONTINUE
C
C--PRINT OUT A MESSAGE IF NOT ALL PARTICLES HAVE BEEN MOVED
C--TO THE END OF TRACKING TIME INTERVAL (TIME2)
C--AT END OF TRANSIENT HEAD SOLUTION
      IF(ISS.EQ.0.AND.TIME2.GT.HT2.AND.NCOUNT.GT.NRC) THEN
        WRITE(IOUT,44)
   44   FORMAT(/1X,'>>END OF TRANSIENT HEAD SOLUTION')
      ENDIF

c-----------------------------------------------------------------------
c v4.6 reverse particle tracking
c
      else
          if (iss.eq.0) then
              call reverseHeads(INH,inbas,NCOL,NROW,NLAY,HDFORM,nper)
              call reverseSS(nper,IOUT,NCOL,NROW,MXWEL,NWELLS,Y(LCWEL),
     +        MXDRN,NDRAIN,Y(LCDRN),MXRIV,NRIVER,Y(LCRIV),MXSTR,NSTREM,
     +        Y(LCSTR),MXGHB,NBOUND,Y(LCGHB),NRCHOP,Y(LCIRCH),Y(LCRECH),
     &        NEVTOP,Y(LCIEVT),Y(LCEVTR),Y(LCEXDP),Y(LCSURF))
          endif
C
C--FOR EACH STRESS PERIOD
      NRC=0
      NF=1
      HT1=0.0
      HT2=0.0
      DO 2000 KPER=NPER,1,-1
          WRITE(*,20) KPER
          WRITE(IOUT,20) KPER
C
C--IF TRANSIENT SIMULATION READ AND PRINT OUT TIMING INFORMATION
          IF(ISS.EQ.0) THEN
              READ(INBAS,'(F10.0,I10.0,F10.0,f10.0)')
     +               PERLEN,NSTP,TSMULT,totim
              WRITE(IOUT,22) PERLEN,NSTP,TSMULT
              IF(TSMULT.EQ.1) THEN
                  DELT=PERLEN/FLOAT(NSTP)
              ELSE
                  DELT=PERLEN*(1.-TSMULT)/(1.-TSMULT**NSTP)
              ENDIF
          ELSE
              NSTP=1
          ENDIF
          if (kper.eq.nper) then                                        !do this step first time along only
              ht2=totim
          endif
C
C--READ AND PREPARE SINK/SOURCE DATA FOR EACH STRESS PERIOD
      CALL GETSS(IOUT,NCOL,NROW,MXWEL,NWELLS,Y(LCWEL),MXDRN,
     & NDRAIN,Y(LCDRN),MXRIV,NRIVER,Y(LCRIV),MXSTR,NSTREM,Y(LCSTR),
     & MXGHB,NBOUND,Y(LCGHB),NRCHOP,Y(LCIRCH),Y(LCRECH),
     & NEVTOP,Y(LCIEVT),Y(LCEVTR),Y(LCEXDP),Y(LCSURF))
C
C--FOR EACH TIME STEP
      DO 2900 KSTP=NSTP,1,-1
          IF(KSTP.GT.1) DELT=DELT*TSMULT
          HT1=HT2
          HT2=HT2-DELT
          WRITE(*,30) KSTP
          WRITE(IOUT,30) KSTP
          IF(ISS.EQ.0) THEN
              WRITE(IOUT,31) HT1,HT2
          ENDIF
C
C--READ IN HEAD DATA FOR EACH TIME STEP
          CALL READH(INH,IOUT,NCOL,NROW,NLAY,Y(LCIB),Y(LCH),HNOFLO,
     &                     HPRT,HDFORM)
C
C--IF STEADY STATE SIMULATION, MOVE ALL PARTICLES DIRECTLY FROM
C--START (TIME1) TO END (TIME2).
          IF(ISS.NE.0) THEN
              TSTART=TIME1
              TEND=TIME2
              GOTO 2100
          ENDIF
c v4.6 - if ITIME option not invoked, then can track particles as done
c        previously, otherwise, need to check particles individually
c        within MOVE subroutine
          if (itime .eq. 0) then                                        !v4.6
C
C--OTHERWISE, MOVE PARTICLES BETWEEN TIME STEPS OF THE HEAD SOLUTION
          print*,'time1=',time1
          print*,'time2=',time2
          print*,'ht1=',ht1
          print*,'ht2=',ht2

              IF(TIME1.le.HT2) THEN
                  WRITE(IOUT,41)
                  GOTO 2900
              ELSEIF(TIME1.lt.HT1.AND.TIME2.gt.HT2) THEN
                  TSTART=TIME1
                  TEND=TIME2
              ELSEIF(TIME1.gE.HT1.AND.TIME2.lT.HT1.AND.TIME2.gT.HT2)THEN
                  TSTART=HT1
                  TEND=TIME2
              ELSEIF(TIME1.lT.HT1.AND.TIME1.gT.HT2.AND.TIME2.lE.HT2)THEN
                  TSTART=TIME1
                  TEND=HT2
              ELSE
                  TSTART=HT1
                  TEND=HT2
              ENDIF
          else                                                          !v4.6
                  tstart=ht1                                            !v4.6
                  tend=ht2                                              !v4.6
          end if                                                        !v4.6
C
C--CALCULATE INTERNODAL DARCY VELOCITIES
 2100     CALL VFACE(NCOL,NROW,NLAY,Y(LCIB),Y(LCH),Y(LCKW),
     &    Y(LCDELR),Y(LCDELC),Y(LCHY),Y(LCTRPY),Y(LCVCON),
     &    MXGHB,NBOUND,Y(LCGHB),NRCHOP,Y(LCIRCH),Y(LCRECH),
     &    NEVTOP,Y(LCIEVT),Y(LCEVTR),Y(LCEXDP),Y(LCSURF),
     &    Y(LCQX),Y(LCQY),Y(LCQZ),idirection)
C
C--MOVE PARTICLES
          CALL MOVE(IOUT,INPLOT,NCOUNT,NRC,NTRKOP,Y(LCPP),Y(LCPI),
     &    TSTART,TEND,EPS,DT,MAXSTP,ISAV,IPRT,NCOL,NROW,NLAY,Y(LCIB),
     &    Y(LCH),Y(LCKW),Y(LCDELR),Y(LCDELC),Y(LCXBC),Y(LCYBC),Y(LCZBC),
     &    Y(LCHTOP),Y(LCDZ),Y(LCPR),XMAX,YMAX,ZMAX,Y(LCQX),Y(LCQY),
     +    Y(LCQZ),ISS,NBOUND,MXGHB,Y(LCGHB),NRIVER,MXRIV,Y(LCRIV),
     +    NSTREM,MXSTR,Y(LCSTR),NDRAIN,MXDRN,Y(LCDRN),NWELLS,MXWEL,
     +    Y(LCWEL),IFRONT,NFRONT,TFRONT,NF,ICAPT,INCAPT,itime,          !v4.6 del NFMAX
     +    nsper,reltime,kper,idirection)                                !v4.6
C
C--CHECK TO SEE WHETHER ALL PARTICLES HAVE ALREADY BEEN REMOVED
C--OR WHETHER IT IS END OF TRACKING TIME INTERVAL
          IF(NRC.EQ.NCOUNT) THEN
              WRITE(IOUT,43)
              GOTO 1001
          ELSEIF(ABS(TEND-TIME2).LT.ROERR) THEN
              WRITE(IOUT,42)
              GOTO 1001
          ENDIF
C
 2900 CONTINUE
 2000 CONTINUE
C
C--PRINT OUT A MESSAGE IF NOT ALL PARTICLES HAVE BEEN MOVED
C--TO THE END OF TRACKING TIME INTERVAL (TIME2)
C--AT END OF TRANSIENT HEAD SOLUTION
      IF(ISS.EQ.0.AND.TIME2.GT.HT2.AND.NCOUNT.GT.NRC) THEN
        WRITE(IOUT,44)
      ENDIF
      end if

C
C--PROGRAM COMPLETED
 1001 STOP
      END
